package asd_library.sorting;


import asd_library.tree.heap.Heap;
/**
 * A collection of static methods implementing a wide variety of sorting algorithms.
 */

public class SortingAlgorithms {

  /**
   * Swaps element in position e1 with that in position e2
   * @param a object array to sort.
   * @param e1
   * @param e2
   */
  protected static void swap(Object[] a, int e1, int e2) {
        Object tmp = a[e1]; a[e1] = a[e2]; a[e2] = tmp;
    }

    /**
     * Scans successive elements for out of order item, then insert the item in the proper place.
     * Sorts small array fast, big array very slowly.
     * @param data array to sort.
     */
    public static void insertionsort(Comparable[] data) {
        Comparable tmp;
        int i, j;
        for (i = 1; i < data.length; i++) {
            tmp = data[i];
            for (j = i; j > 0 && tmp.compareTo(data[j-1]) < 0; j--)
                data[j] = data[j-1];
            data[j] = tmp;
        }
    }

    /**
     * Finds the least element in the array, and put it in the proper place.
     * Repeats until array is sorted.
     * @param data array to sort.
     */
    public static void selectionsort(Comparable[] data) {
        int i,j,least;
        for (i = 0; i < data.length-1; i++) {
            for (j = i+1, least = i; j < data.length; j++)
                if (data[j].compareTo(data[least]) < 0)
                    least = j;
            swap(data,least,i);
        }
    }

    /**
     * Partitions array into two segments: the first segment all elements are less
     * than or equal to the pivot value (bound), the second segment all elements are
     * greater or equal to the pivot value. Sorts the two segments recursively.
     * Quicksort is fastest on average, but sometimes unbalanced partitions can
     * lead to very slow sorting.
     * @param data array which the segment to order belongs to.
     * @param first is the index of the first element in the segment.
     * @param last is the index of the last element in the segment.
     */
    public static void quicksort(Comparable[] data, int first, int last) {
        int lower = first + 1, upper = last;
        swap(data,first,(first+last)/2);
        Comparable bound = data[first];
        while (lower <= upper) {
            while (data[lower].compareTo(bound) < 0)
                 lower++;
            while (bound.compareTo(data[upper]) < 0)
                 upper--;
            if (lower < upper)
                 swap(data,lower++,upper--);
            else lower++;
        }
        swap(data,upper,first);
        if (first < upper-1)
            quicksort(data,first,upper-1);
        if (upper+1 < last)
            quicksort(data,upper+1,last);
    }

    /**
     * Finds the largest element and put it at the end of data then calls the
     * recursive method <b>quicksort</b>(Comparable[] data, int first, int last)
     * @param data array to sort.
     */
    public static void quicksort(Comparable[] data) {
        if (data.length < 2)
            return;
        int max = 0;
        // find the largest element and put it at the end of data;
        // Just a trick to simplify code....;
        for (int i = 1; i < data.length; i++)
            if (data[max].compareTo(data[i]) < 0)
                max = i;
        swap(data,data.length-1,max);    // largest el is now in its
        quicksort(data,0,data.length-2); // final position;
    }

    /**
     * Scans successive elements for out of order item, then insert the item in
     * the proper place of the segment between the index first and last. Is called
     * by the method <b>quicksort2</b>(Comparable[] data, int first, int last).
     * @param data array which the segment to order belongs to.
     * @param first is the index of the first element in the segment.
     * @param last is the index of the last element in the segment.
     */
    public static void insertionsort(Comparable[] data, int first, int last) {
        Comparable tmp;
        int i, j;
        for (i = first; i <= last; i++) {
            tmp = data[i];
            for (j = i; j > 0 && tmp.compareTo(data[j-1]) < 0; j--)
                data[j] = data[j-1];
            data[j] = tmp;
        }
    }


    /**
     * quicksort2 uses insertionsort in recursion when size of sub-array < 30.
     * @param data data array which the segment to order belongs to.
     * @param first is the index of the first element in the segment.
     * @param last is the index of the last element in the segment.
     */
    public static void quicksort2(Comparable[] data, int first, int last) {
        if (last - first < 30)
             insertionsort(data,first,last);
        else {
             int lower = first + 1, upper = last;
             swap(data,first,(first+last)/2);
             Comparable bound = data[first];
             while (lower <= upper) {
                 while (data[lower].compareTo(bound) < 0)
                     lower++;
                 while (bound.compareTo(data[upper]) < 0)
                     upper--;
                 if (lower < upper)
                     swap(data,lower++,upper--);
                 else lower++;
             }
             swap(data,upper,first);
             if (first < upper-1)
                  quicksort2(data,first,upper-1);
             if (upper+1 < last)
                  quicksort2(data,upper+1,last);
        }
    }


    /**
     * Finds the largest element and puts it at the end of data then calls the
     * method <b>quicksort2</b>(Comparable[] data, int first, int last).
     * @param data array to sort.
     */
    public static void quicksort2(Comparable[] data) {
        if (data.length < 2)
            return;
        int max = 0;
        // find the largest element and put it at the end of data;
        // Just a trick to simplify code....;
        for (int i = 1; i < data.length; i++)
            if (data[max].compareTo(data[i]) < 0)
                max = i;
        swap(data,data.length-1,max);     // largest el is now in its
        quicksort2(data,0,data.length-2); // final position;
    }

    /**
     * static variable used by merge();
     */
    static Comparable[] temp;

    /**
     * Merges the two sorted halves of a segment into a only one ordered segment.
     * @param data array which the segment to order belongs to.
     * @param first is the index of the first element in the segment.
     * @param last is the index of the last element in the segment.
     * */

    protected static void merge(Comparable[] data, int first, int last) {
        int mid = (first + last) / 2;
        int i1 = 0, i2 = first, i3 = mid + 1;
        while (i2 <= mid && i3 <= last)
            if (data[i2].compareTo(data[i3]) < 0)
                 temp[i1++] = data[i2++];
            else temp[i1++] = data[i3++];
        while (i2 <= mid)
            temp[i1++] = data[i2++];
        while (i3 <= last)
            temp[i1++] = data[i3++];
        for (i1 = 0, i2 = first; i2 <= last; data[i2++] = temp[i1++]);
    }

    /**
     * Divides recursively the array to order in two half up to find segments of
     * length 1. Therefore calls the <b>merge</b>(Comparable[] data, int first, int last)
     * method which merge into a single run of twice the length.
     * @param data array which the segment to order belongs to.
     * @param first is the index of the first element in the segment.
     * @param last is the index of the last element in the segment.
     */
    public static void mergesort(Comparable[] data, int first, int last) {
        if (first < last) {
            int mid = (first + last) / 2;
            mergesort(data, first, mid);
            mergesort(data, mid+1, last);
            merge(data, first, last);
        }
    }

    /**
     * Calls the recursive method <b>mergesort</b>(Comparable[] data, int first, int last)
     * @param data array to sort.
     */
    public static void mergesort(Comparable[] data) {
        temp = new Comparable[data.length];
        mergesort(data,0,data.length-1);
    }



}